Module: LoadedDie

Defined in:
lib/loaded_die.rb

Overview

This module provides a class for randomly choosing from a set of options where some options are more likely than others.

Defined Under Namespace

Classes: Sampler

Constant Summary collapse

VERSION =

The version string.

'1.0.2'